<?php
require_once('gce-feed.php');

class GCE_Parser{
	var $feeds = array();
	var $merged_feed_data = array();
	var $title = null;
	var $max_events_display = 0;

	function GCE_Parser($feed_ids, $title_text = null, $max_events = 0){
		$this->__construct($feed_ids, $title_text, $max_events);
	}

	function __construct($feed_ids, $title_text = null, $max_events = 0){
		$this->title = $title_text;
		$this->max_events_display = $max_events;

		//Get the feed options
		$options = get_option(GCE_OPTIONS_NAME);

		foreach($feed_ids as $single_feed){
			//Get the options for this particular feed
			if(isset($options[$single_feed])){
				$feed_options = $options[$single_feed];

				$feed = new GCE_Feed();

				$feed->set_feed_id($feed_options['id']);
				$feed->set_feed_url($feed_options['url']);
				$feed->set_max_events($feed_options['max_events']);
				//If day limit is not blank, set end date to specified number of days in the future, including today
				if($feed_options['day_limit'] != '') $feed->set_end_date(mktime(0, 0, 0, date('n'), date('j'), date('Y')) + (86400 * $feed_options['day_limit']));
				$feed->set_cache_duration($feed_options['cache_duration']);
				//Set the timezone if anything other than default
				if($feed_options['timezone'] != 'default') $feed->set_timezone($feed_options['timezone']);
				//If show past events is true, set start date to 1st of this month. Otherwise, set start date to today. (Adjust for timezone)
				if($feed_options['show_past_events'] == 'true'){
					//$feed->set_start_date(mktime(0, 0, 0, date('m'), 1, date('Y')) - date('Z'));
					$feed->set_start_date(mktime(0, 0, 0, date('m'), date('j'), date('Y')) - date('Z') - 86400*(date('N')-1)  );
				}else{
					$feed->set_start_date(mktime(0, 0, 0, date('m'), date('j'), date('Y')) - date('Z'));
				}
				//Set date and time formats. If they have not been set by user, set to global WordPress formats 
				$feed->set_date_format($feed_options['date_format'] == '' ? get_option('date_format') : $feed_options['date_format']);
				$feed->set_time_format($feed_options['time_format'] == '' ? get_option('time_format') : $feed_options['time_format']);
				//Set whether to handle multiple day events
				$feed->set_multi_day($feed_options['multiple_day'] == 'true' ? true : false);

				//Sets all display options
				$feed->set_display_options(array(
					'display_color' => $feed_options['display_color'],
					'display_start' => $feed_options['display_start'],
					'display_end' => $feed_options['display_end'],
					'display_location' => $feed_options['display_location'],
					'display_desc' => $feed_options['display_desc'],
					'display_link' => $feed_options['display_link'],
					'display_start_text' => $feed_options['display_start_text'],
					'display_end_text' => $feed_options['display_end_text'],
					'display_location_text' => $feed_options['display_location_text'],
					'display_desc_text' => $feed_options['display_desc_text'],
					'display_desc_limit' => $feed_options['display_desc_limit'],
					'display_link_text' => $feed_options['display_link_text'],
					'display_link_target' => $feed_options['display_link_target'],
					'display_separator' => $feed_options['display_separator']
				));

				//SimplePie does the hard work
				$feed->init();

				//Add feed object to array of feeds
				$this->feeds[$single_feed] = $feed;
			}
		}

		//More SimplePie magic to merge items from all feeds together
		$this->merged_feed_data = SimplePie::merge_items($this->feeds);

		//Sort the items by into date order
		usort($this->merged_feed_data, array('SimplePie_Item_GCalendar', 'compare'));
	}

	//Returns an array of feed ids that have encountered errors
	function get_errors(){
		$errors = array();

		foreach($this->feeds as $feed){
			//Remove '//' on line below to see more error information
			if($feed->error()) $errors[] = $feed->get_feed_id();
		}

		return $errors;
	}

	//Returns array of days with events, with sub-arrays of events for that day
	function get_event_days(){
		$event_days = array();

		//Total number of events retrieved
		$count = count($this->merged_feed_data);

		//If maximum events to display is 0 (unlimited) set $max to 1, otherwise use maximum of events specified by user
		$max = $this->max_events_display == 0 ? 1 : $this->max_events_display;

		//Loop through entire array of events, or until maximum number of events to be displayed has been reached
		for($i = 0; $i < $count && $max > 0; $i++){
			$item = $this->merged_feed_data[$i];

			//Check that event end time isn't before start time of feed (ignores events from before start time that may have been inadvertently retrieved)
			if($item->get_end_date() > ($item->get_feed()->get_start_date() + date('Z'))){

				$start_date = $item->get_start_date();

				//Round start date to nearest day
				$start_date = mktime(0, 0, 0, date('m', $start_date), date('d', $start_date) , date('Y', $start_date));

				//If multiple day events should be handled, add multiple day event to required days
				if($item->get_feed()->get_multi_day()){
					$on_next_day = true;
					$next_day = $start_date + 86400;
					while($on_next_day){
						if($item->get_end_date() > $next_day){
							$event_days[$next_day][] = $item;
						}else{
							$on_next_day = false;
						}
						$next_day += 86400;
					}
				}

				//Add item into array of events for that day
				$event_days[$start_date][] = $item;

				//If maximum events to display isn't 0 (unlimited) decrement $max counter
				if($this->max_events_display != 0) $max--;
			}
		}

		return $event_days;
	}
	
	function sort_array_by_first_int_index($my_array){
		foreach ($my_array as $key => $sub_array){
			$indexes_array[] = $key;
		}
		sort($indexes_array);
		foreach ($indexes_array as $int_value){
			$result_array[$int_value] = $my_array[$int_value];
		}
		return $result_array;
	}
	
	//Returns array of hours with events, with sub-arrays of week days for each hour
	function get_event_days_table(){
		$event_hours_days_table = array();

		//Total number of events retrieved
		$count = count($this->merged_feed_data);

		//If maximum events to display is 0 (unlimited) set $max to 1, otherwise use maximum of events specified by user
		$max = $this->max_events_display == 0 ? 1 : $this->max_events_display;

		//Loop through entire array of events, or until maximum number of events to be displayed has been reached
		for($i = 0; $i < $count && $max > 0; $i++){
			$item = $this->merged_feed_data[$i];

			//Check that event end time isn't before start time of feed (ignores events from before start time that may have been inadvertently retrieved)
			if($item->get_end_date() > ($item->get_feed()->get_start_date() + date('Z'))){

				$start_date = $item->get_start_date();

				//Round start date to nearest day
				//$start_date = mktime(0, 0, 0, date('m', $start_date), date('d', $start_date) , date('Y', $start_date));
				//$event_rounded_hour = date('H', $start_date);
				$event_week_day = date('N', $start_date);
				$event_time = date('H', $start_date).':'.date('i', $start_date);
				$display_options = $item->get_feed()->get_display_options();
				//Add item into array of events for that day
				$event_hours_days_table[$event_time][$event_week_day] = array('title'  => $item->get_title(),
																			  'ftitle' => $item->get_feed()->get_title(),
																			  'color' => $display_options['display_color'],
																			  'timestamp' => $start_date);															  

				//If maximum events to display isn't 0 (unlimited) decrement $max counter
				if($this->max_events_display != 0) $max--;
			}
		}
		
		$event_hours_days_table = $this->sort_array_by_first_int_index($event_hours_days_table);
		//Print_r($event_hours_days_table);
		return $event_hours_days_table;
		
	}

	
	//Return weekly timetable
	function get_table($grouped = false, $week_days_names, $hour12){
		
		//$event_days = $this->get_event_days();
		
		$event_days_table = $this->get_event_days_table();

		//If event_days is empty, there are no events in the feed(s), so return a message indicating this
		if(count((array)$event_days_table) == 0) return '<p>' . __('There are currently no upcoming events.', GCE_TEXT_DOMAIN) . '</p>';

		$today = mktime(0, 0, 0, date('m'), date('d'), date('Y'));
		
		//$indexed_edt = array_values($event_days_table);
		//Print_r($indexed_edt);
		$number_of_days = 6;
		foreach($event_days_table as $key => $events_time){
			if(isset($events_time[7])){
				$number_of_days = 7;
			}
		}

		$markup = '<table class="pretty-table"><tr><th scope="col"></th>';
		for($i=0;$i < $number_of_days; $i++){
			$markup .= '<th scope="col">'.$week_days_names[$i].'</th>';
				
		}
		
				
		foreach($event_days_table as $key => $events_time){
			//calculate hours differently if it's 12-hour or 24-hour clock. 24hour clock is by default
			if($hour12 == false){
			$markup .= '</tr><tr>
			<th scope="row">'.$key.'</th>';
			} else {
			$indexed_arr = array_values($events_time);
			$timestamp = $indexed_arr[0]['timestamp'];
			$time_string = date('g', $timestamp).':'.date('i', $timestamp).date('a', $timestamp);
			$markup .= '</tr><tr>
			<th scope="row">'.$time_string.'</th>';
			}

			
			for($i = 1; $i <= $number_of_days; $i++){
				//Create the markup for this event
				$markup .=
					'<td style="background:'.$events_time[$i]['color'].'">' .
					$events_time[$i]['title'].
					'</td>';
	
			}
			
			$markup .= '</tr>';
		}
		$markup .= '</table>';
		
		return $markup;
	}

	//Returns the event information markup for the specified event (type = tooltip or list)
	function get_event_info_markup($event, $type){
		//Get the feed from which this event comes
		$feed = $event->get_feed();

		$display_options = $feed->get_display_options();

		$markup = '';

		$start_end = array();

		//If start date / time should be displayed, set up array of start date and time
		if($display_options['display_start'] != 'none'){
			$sd = $event->get_start_date();
			$start_end['start'] = array('time' => date_i18n($feed->get_time_format(), $sd), 'date' => date_i18n($feed->get_date_format(), $sd));
		}

		//If end date / time should be displayed, set up array of end date and time
		if($display_options['display_end'] != 'none'){
			$ed = $event->get_end_date();
			$start_end['end'] = array('time' => date_i18n($feed->get_time_format(), $ed), 'date' => date_i18n($feed->get_date_format(), $ed));
		}

		//Add the correct start / end, date / time information to $markup
		foreach($start_end as $start_or_end => $info){
			$markup .= '<p class="gce-' . $type . '-' . $start_or_end . '"><span>' . $display_options['display_' . $start_or_end . '_text'] . '</span> ';

			switch($display_options['display_' . $start_or_end]){
				case 'time': $markup .= $info['time'];
					break;
				case 'date': $markup .= $info['date'];
					break;
				case 'time-date': $markup .= $info['time'] . $display_options['display_separator'] . $info['date'];
					break;
				case 'date-time': $markup .= $info['date'] . $display_options['display_separator'] . $info['time'];
			}

			$markup .= '</p>';
		}

		//If location should be displayed (and is not empty) add to $markup
		if(isset($display_options['display_location'])){
			$event_location = $event->get_location();
			if($event_location != '') $markup .= '<p class="gce-' . $type . '-loc"><span>' . $display_options['display_location_text'] . '</span> ' . esc_html($event_location) . '</p>';
		}

		//If description should be displayed (and is not empty) add to $markup
		if(isset($display_options['display_desc'])){
			$event_desc = $event->get_description();

			if($event_desc != ''){
				//Limit number of words of desc to display, if required
				if($display_options['display_desc_limit'] != ''){
					preg_match('/([\S]+\s*){0,' . $display_options['display_desc_limit'] . '}/', $event->get_description(), $event_desc);
					$event_desc = trim($event_desc[0]);
				}

				$markup .= '<p class="gce-' . $type . '-desc"><span>' . $display_options['display_desc_text'] . '</span> ' . make_clickable(nl2br(esc_html($event_desc))) . '</p>';
			}
		}

		//If link should be displayed add to $markup
		if(isset($display_options['display_link'])){                                                                                   //Below: add target="_blank" if required
			$markup .= '<p class="gce-' . $type . '-link"><a href="' . $event->get_link() . '&amp;ctz=' . $feed->get_timezone() . '"' . (isset($display_options['display_link_target']) ? ' target="_blank"' : '') . '>' . $display_options['display_link_text'] . '</a></p>';
		}

		return $markup;
	}
}
?>